#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
SPAM Rados Module
"""

import spam.ansirunner
import re


class Rados(object):
    """
    SPAM Rados class
    """
    def __init__(self):
        """
        Initialize Rados class.
        """
        self.runner = spam.ansirunner.AnsibleRunner()

    def rados_df(self,
                 host_list=None,
                 remote_user=None,
                 remote_pass=None):
        '''
        Invoked the rados df command and return output to user
        '''
        result, failed_hosts = self.runner.ansible_perform_operation(
            host_list=host_list,
            remote_user=remote_user,
            remote_pass=remote_pass,
            module="command",
            module_args="rados df")

        parsed_result = self.rados_parse_df(result)

        return parsed_result

    def rados_parse_df(self,
                       result):
        '''
        Parse the result from ansirunner module and save it as a json
        object
        '''
        parsed_results = []
        HEADING = r".*(pool name) *(category) *(KB) *(objects) *(clones)" + \
            " *(degraded) *(unfound) *(rd) *(rd KB) *(wr) *(wr KB)"

        HEADING_RE = re.compile(HEADING,
                                re.IGNORECASE)

        dict_keys = ["pool_name", "category", "size_kb", "objects",
                     "clones", "degraded", "unfound", "rd", "rd_kb",
                     "wr", "wr_kb"]

        if result['contacted'].keys():
            for node in result['contacted'].keys():
                df_result = {}
                nodeobj = result['contacted'][node]
                df_output = nodeobj['stdout']
                for line in df_output.splitlines():
                    print "Line: ", line
                    # Skip the heading line.
                    reobj = HEADING_RE.match(line)
                    if not reobj:
                        row = line.split()
                        if len(row) != len(dict_keys):
                            print "line not match: ", line
                            continue

                        key_count = 0
                        for column in row:
                            df_result[dict_keys[key_count]] = column
                            key_count += 1
                        print "df_result: ", df_result
                        parsed_results.append(df_result)

                    nodeobj['parsed_results'] = parsed_results

        return result




#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
AnsibleInventory:

INTRO:

USAGE:

"""

import os
import ansible.inventory


class AnsibleInventory(object):
    '''
    Ansible Inventory wrapper class.
    '''
    def __init__(self, inventory_filename):
        '''
        Initialize Inventory
        '''
        self.inventory = None
        if not os.path.exists(inventory_filename):
            print "Provide a valid inventory filename"
            return

        self.inventory = ansible.inventory.InventoryParser(inventory_filename)

    def get_hosts(self, group=None):
        '''
        Get the hosts
        '''
        hostlist = []

        if group:
            groupobj = self.inventory.groups.get(group)
            if not groupobj:
                print "Group [%s] not found in inventory" % group
                return None

            groupdict = {}
            groupdict['hostlist'] = []
            for host in groupobj.get_hosts():
                groupdict['hostlist'].append(host.name)
            hostlist.append(groupdict)
        else:
            for group in self.inventory.groups:
                groupdict = {}
                groupdict['group'] = group
                groupdict['hostlist'] = []
                groupobj = self.inventory.groups.get(group)
                for host in groupobj.get_hosts():
                    groupdict['hostlist'].append(host.name)
                hostlist.append(groupdict)

        return hostlist






#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
AnsibleRunner:

INTRO:

USAGE:

"""

import ansible.runner


class AnsibleRunner(object):
    '''
    Ansible Runner Wrapper class.
    '''
    def __init__(self, host_list=None, remote_user=None):
        '''
        Initialize AnsibleRunner.
        '''
        self.host_list = host_list
        self.remote_user = remote_user

    def validate_host_parameters(self, host_list, remote_user):
        '''
        Validate and set the host list and remote user parameters.
        '''
        if host_list is None:
            host_list = self.host_list

        if remote_user is None:
            remote_user = self.remote_user

        if host_list is None or remote_user is None:
            print "Host list [%s], remote user [%s] are required" % \
                  (host_list, remote_user)
            return (None, None)

        return (host_list, remote_user)

    def validate_results(self, results, checks=None):
        '''
        Valdiate results from the Anisble Run.
        '''
        results['status'] = 'PASS'
        failed_hosts = []

        ###################################################
        # First validation is to make sure connectivity to
        # all the hosts was ok.
        ###################################################
        if results['dark']:
            print "Host connectivity issues on %s",
            results['dark'].keys()
            failed_hosts.append(results['dark'].keys())
            results['status'] = 'FAIL'

        ##################################################
        # Now look for status 'failed'
        ##################################################
        for node in results['contacted'].keys():
            if 'failed' in results['contacted'][node]:
                if results['contacted'][node]['failed'] is True:
                    results['status'] = 'FAIL'

        #################################################
        # Check for the return code 'rc' for each host.
        #################################################
        for node in results['contacted'].keys():
            rc = results['contacted'][node].get('rc', None)
            if rc is not None and rc != 0:
                print "Operation 'return code' %s on host %s" % \
                    (results['contacted'][node]['rc'], node)
                failed_hosts.append(node)
                results['status'] = 'FAIL'

        ##################################################
        # Additional checks. If passed is a list of key/value
        # pairs that should be matched.
        ##################################################
        if checks is None:
            #print "No additional checks validated"
            return results, failed_hosts

        for check in checks:
            key = check.keys()[0]
            value = check.values()[0]
            for node in results['contacted'].keys():
                if key in results['contacted'][node].keys():
                    if results['contacted'][node][key] != value:
                        failed_hosts.append(node)
                        results['status'] = 'FAIL'

        return (results, failed_hosts)

    def ansible_perform_operation(self,
                                  host_list=None,
                                  remote_user=None,
                                  remote_pass=None,
                                  module=None,
                                  complex_args=None,
                                  module_args='',
                                  environment=None,
                                  check=False,
                                  sudo=False,
                                  sudo_user=None,
                                  sudo_pass=None,
                                  forks=20):
        '''
        Perform any ansible operation.
        '''
        (host_list, remote_user) = \
            self.validate_host_parameters(host_list, remote_user)
        if (host_list, remote_user) is (None, None):
            return None

        if module is None:
            print "ANSIBLE Perform operation: No module specified"
            return None

        runner = ansible.runner.Runner(
            module_name=module,
            host_list=host_list,
            remote_user=remote_user,
            remote_pass=remote_pass,
            module_args=module_args,
            complex_args=complex_args,
            environment=environment,
            check=check,
            become=sudo,
            become_user=sudo_user,
            become_pass=sudo_pass,
            forks=forks)

        results = runner.run()

        results, failed_hosts = self.validate_results(results)
        if results['status'] != 'PASS':
            print "ANSIBLE: [%s] operation failed [%s] [hosts: %s]" % \
                (module, complex_args, failed_hosts)

        return results, failed_hosts

    def ansible_get_dark_hosts(self,
                               ansible_result):
        '''
        Given the ansible result object, return the list of
        hosts that could not be contacted
        '''
        if not ansible_result['dark']:
            return []

        return ansible_result['dark'].keys()







#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
SPAM virsh Module
The module interfaces with virsh commands to get hypervisor and VM stats.
"""

import spam.ansirunner
import rex


class Virsh(object):
    """
    SPAM Virsh class
    """
    def __init__(self,
                 host_list=None,
                 remote_user=None,
                 remote_pass=None,
                 sudo=False,
                 sudo_user=None,
                 sudo_pass=None):
        '''
        Initialize virsh.
        '''
        self.host_list = host_list
        self.remote_user = remote_user
        self.remote_pass = remote_pass
        self.sudo = sudo
        self.sudo_user = sudo_user
        self.sudo_pass = sudo_pass
        self.runner = spam.ansirunner.AnsibleRunner()

    def get_validated_params(self,
                             host_list, remote_user, remote_pass,
                             sudo, sudo_user, sudo_pass):
        if host_list is None:
            host_list = self.host_list

        if remote_user is None:
            remote_user = self.remote_user

        if remote_pass is None:
            remote_pass = self.remote_pass

        if sudo:
            if sudo_user is None:
                sudo_user = self.sudo_user
            if sudo_pass is None:
                sudo_pass = self.sudo_pass

        return(host_list, remote_user, remote_pass,
               sudo, sudo_user, sudo_pass)

    def execute_virsh_command(self,
                              **kwargs):
        '''
        common virsh execution function
        '''
        host_list = kwargs.get('host_list', None)
        remote_user = kwargs.get('remote_user', None)
        remote_pass = kwargs.get('remote_pass', None)
        sudo = kwargs.get('sudo', False)
        sudo_user = kwargs.get('sudo_user', None)
        sudo_pass = kwargs.get('sudo_pass', None)

        host_list, remote_user, remote_pass, \
            sudo, sudo_user, sudo_pass = self.get_validated_params(
                host_list, remote_user,
                remote_pass, sudo,
                sudo_user, sudo_pass)

        if 'cmd' not in kwargs.keys():
            print "Require a command to execute"
            return None
        cmd = kwargs['cmd']

        if 'delimiter' not in kwargs.keys():
            delimiter = ":"
        else:
            delimiter = kwargs['delimiter']

        if 'output_type' not in kwargs.keys():
            output_type = "LRVALUE"
        else:
            output_type = kwargs['output_type']

        if output_type == "TABLE":
            if 'fields' not in kwargs.keys():
                print "Require to pass fields"
                return None

        fields = kwargs['fields']

        result, failed_hosts = self.runner.ansible_perform_operation(
            host_list=host_list,
            remote_user=remote_user,
            remote_pass=remote_pass,
            module="command",
            module_args=cmd,
            sudo=sudo,
            sudo_user=sudo_user,
            sudo_pass=sudo_pass)

        virsh_result = None
        if result['contacted'].keys():
            virsh_result = {}
            for node in result['contacted'].keys():
                nodeobj = result['contacted'][node]
                if output_type == "LRVALUE":
                    jsonoutput = rex.parse_lrvalue_string(nodeobj['stdout'],
                                                          delimiter)
                elif output_type == "TABLE":
                    jsonoutput = rex.parse_tabular_string(nodeobj['stdout'],
                                                          fields)
                else:
                    pass
                virsh_result[node] = {}
                virsh_result[node]['result'] = jsonoutput

        return virsh_result

    def virsh_version(self,
                      host_list=None,
                      remote_user=None,
                      remote_pass=None,
                      sudo=False,
                      sudo_user=None,
                      sudo_pass=None):
        '''
        Get the virsh version
        '''
        host_list, remote_user, remote_pass, \
            sudo, sudo_user, sudo_pass = self.get_validated_params(
               host_list, remote_user, remote_pass, sudo, sudo_user,
               sudo_pass)


        result, failed_hosts = self.runner.ansible_perform_operation(
            host_list=host_list,
            remote_user=remote_user,
            remote_pass=remote_pass,
            module="command",
            module_args="virsh version",
            sudo=sudo,
            sudo_user=sudo_user,
            sudo_pass=sudo_pass)

        virsh_result = None

        if result['contacted'].keys():
            virsh_result = {}
            for node in result['contacted'].keys():
                nodeobj = result['contacted'][node]
                jsonoutput = rex.parse_lrvalue_string(nodeobj['stdout'], ":")
                virsh_result[node] = {}
                virsh_result[node]['result'] = jsonoutput

        return virsh_result

    def virsh_list(self,
                   host_list=None,
                   remote_user=None,
                   remote_pass=None,
                   sudo=False,
                   sudo_user=None,
                   sudo_pass=None):
        '''
        Get virsh list --state-running
        <TODO>: Add an options argument for different options to
        collect the virsh list output. Need a fix to  rex module
        tabular before that.
        '''
        host_list, remote_user, remote_pass, \
            sudo, sudo_user, sudo_pass = self.get_validated_params(
                       host_list, remote_user, remote_pass, sudo, sudo_user,
                       sudo_pass)

        result, failed_hosts = self.runner.ansible_perform_operation(
            host_list=host_list,
            remote_user=remote_user,
            remote_pass=remote_pass,
            module="command",
            module_args="virsh list --state-running",
            sudo=sudo,
            sudo_user=sudo_user,
            sudo_pass=sudo_pass)

        virsh_result = None

        fields = ["Id", "Name", "state"]

        if result['contacted'].keys():
            virsh_result = {}
            for node in result['contacted'].keys():
                nodeobj = result['contacted'][node]
                jsonoutput = rex.parse_tabular_string(nodeobj['stdout'],
                                                      fields)
                virsh_result[node] = {}
                virsh_result[node]['result'] = jsonoutput

        return virsh_result

    def virsh_per_domain_info(self,
                              **kwargs):
        '''
        Get per domain stats from each hosts passed as hostlist.
        '''
        host_list = kwargs.get('host_list', self.host_list)
        remote_user = kwargs.get('remote_user', self.remote_user)
        remote_pass = kwargs.get('remote_pass', self.remote_pass)
        sudo = kwargs.get('sudo', self.sudo)
        sudo_user = kwargs.get('sudo_user', self.sudo_user)
        sudo_pass = kwargs.get('sudo_pass', self.sudo_pass)

        result, failed_hosts = self.runner.ansible_perform_operation(
            host_list=host_list,
            remote_user=remote_user,
            remote_pass=remote_pass,
            module="command",
            module_args="virsh list",
            sudo=sudo,
            sudo_user=sudo_user,
            sudo_pass=sudo_pass)

        virsh_result = None

        fields = ["Id", "Name", "state"]

        if result['contacted'].keys():
            virsh_result = {}
            for node in result['contacted'].keys():
                nodeobj = result['contacted'][node]
                jsonoutput = rex.parse_tabular_string(nodeobj['stdout'],
                                                      fields)
                virsh_result[node] = {}
                virsh_result[node]['result'] = jsonoutput

        print virsh_result


        return virsh_result





from setuptools import setup, find_packages

with open('README.md') as fhandle:
    long_description = fhandle.read()


setup(
    name='pyansible',
    version='1.0.1',
    description='A module for interfacing with Ansible Runner and Inventory.',
    long_description=long_description,
    url="https://github.com/bdastur/spam",
    author="Behzad Dastur",
    author_email="bdastur@gmail.com",
    license='Apache Software License',
    classifier=[
        'Development Status :: 3 - Alpha',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Intended Audience :: Developers',
        'License :: Apache Software License',
        'Topic :: Software Development :: Libraries',
        'Topic :: Utilities',
        'License :: OSI Approved :: Apache Software License',
        'Operating System :: POSIX :: Linux',
        'Operating System :: MacOS',
    ],

    keywords='ansible',
    packages=find_packages(exclude=['contrib', 'docs', 'tests']),
    install_requires=['rexutil',
                      'MarkupSafe',
                      'jinja2',
                      'PyYAML',
                      'ansible==1.9.4']
)

